﻿using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFPocoCSCreater.DataBaseHelper;
using EFPocoCSCreater.UseModel;

namespace EFPocoCSCreater.CreateBusiness
{
    public class Businesser
    {
        #region 变量

        private IDataBase dataBase;

        #endregion

        #region 对外方法
        public bool InitDataBase(string ip, string catalog, string user, string pass)
        {
            dataBase = DataBaseFactory.CreateDataBase(ip, catalog, user, pass);
            return dataBase.CanOpen;
        }

        /// <summary>
        /// 获取所有的表
        /// </summary>
        /// <returns></returns>
        public List<string> GetDataTable()
        {
            string sql = "Select Name FROM SysObjects Where XType='U' orDER BY Name";
            string error;
            DataTable dt = dataBase.GetDataTable(sql, out error);
            List<string> data = new List<string>();
            foreach (DataRow dr in dt.Rows)
            {
                data.Add(dr[1].ToString());
            }
            return data;
        }
        /// <summary>
        /// 获取所有的视图
        /// </summary>
        /// <returns></returns>
        public List<string> GetDataView()
        {
            string sql = "Select Name FROM SysObjects Where XType='V' orDER BY Name";
            string error;
            DataTable dt = dataBase.GetDataTable(sql, out error);
            List<string> data = new List<string>();
            foreach (DataRow dr in dt.Rows)
            {
                data.Add(dr[1].ToString());
            }
            return data;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="tables"></param>
        /// <returns></returns>
        public List<TableInfo> GetTableInfoForCreate(List<string> tables)
        {
            List<TableInfo> data = new List<TableInfo>();

            foreach (var item in tables)
            {
                string sql = string.Format("SELECT syscolumns.name as columName,systypes.name as columType,syscolumns.isnullable,syscolumns.length FROM syscolumns, systypes "
                        + " WHERE syscolumns.xusertype = systypes.xusertype AND syscolumns.id = object_id('{0}')", item);
                string error;
                DataTable dt = dataBase.GetDataTable(sql, out error);
                var table = new TableInfo { Name = item, Fields = new List<FieldInfo>() };
                foreach (DataRow dr in dt.Rows)
                {
                    var fieldName = dr["columName"].ToString();
                    var fieldType = dr["columType"].ToString();
                    var filedNull = dr["isnullable"].ToString();
                    var filedlen = int.Parse(dr["length"].ToString());
                    //主键就不生成
                    if (fieldName.ToUpper() == "ID") continue;
                    var field = new FieldInfo
                                    {
                                        DataLength = filedlen,
                                        DataType = fieldType,
                                        IsNull = filedNull == "1",
                                        Name = fieldName
                                    };

                    field.CsDataType = DataTypeTool.SqlTypeToCsType(fieldType, field.IsNull);
                    table.Fields.Add(field);
                }
                data.Add(table);
            }


            return data;
        }

        #endregion
    }

    public class DataTypeTool
    {
        public static string SqlTypeToCsType(string type,bool isnull)
        {
            var key = type.ToLower();
            var val = string.Empty;
            switch (key)
            {
                case "char":
                    {
                        val = "string";
                    }
                    break;
                case "varchar":
                    {
                        val = "string";
                    }
                    break;
                case "nchar":
                    {
                        val = "string";
                    }
                    break;
                case "nvarchar":
                    {
                        val = "string";
                    }
                    break;
                case "int":
                    {
                        val = "int";
                        if (isnull)
                            val += "?";
                    }
                    break;
                case "decimal":
                    {
                        val = "decimal";
                        if (isnull)
                            val += "?";
                    }
                    break;
                case "numeric":
                    {
                        val = "float";
                        if (isnull)
                            val += "?";
                    }
                    break;
                case "float":
                    {
                        val = "float";
                        if (isnull)
                            val += "?";
                    }
                    break;
                case "money":
                    {
                        val = "float";
                        if (isnull)
                            val += "?";
                    }
                    break;
                case "datetime":
                    {
                        val = "DateTime";
                        if (isnull)
                            val += "?";
                    }
                    break;
                case "date":
                    {
                        val = "DateTime";
                        if (isnull)
                            val += "?";
                    }
                    break;
                case "time":
                    {
                        val = "DateTime";
                        if (isnull)
                            val += "?";
                    }
                    break;
                case "bit":
                    {
                        val = "bool";
                        if (isnull)
                            val += "?";
                    }
                    break;
            }
            return val;
        }
    }
}
