﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using EFPocoCSCreater.CreateBusiness;
using EFPocoCSCreater.UseModel;
using OMS.Ice.T4Generator;

namespace EFPocoCSCreater.WinUI
{
    public partial class FrmMain : Form
    {
        #region 变量

        private Businesser businesser;

        #endregion

        #region 初始化
        public FrmMain()
        {
            InitializeComponent();

            businesser = new Businesser();
        }
        #endregion

        #region 连接数据库
        private void btnConnect_Click(object sender, EventArgs e)
        {
            var ip = tbxServerIp.Text;
            var name = tbxDataBaseName.Text;
            var user = tbxUserName.Text;
            var pass = tbxPassword.Text;

            if (ip.Length == 0)
            {
                return;
            }
            if (name.Length == 0)
            {
                return;
            }
            if (user.Length == 0)
            {
                return;
            }
            if (pass.Length == 0)
            {
                return;
            }

            btnConnect.Enabled = !businesser.InitDataBase(ip, name, user, pass);

            if (btnConnect.Enabled) return;

            var tableData = businesser.GetDataTable();
            var viewData = businesser.GetDataView();

            listTable.Items.Clear();
            foreach (var item in tableData)
            {
                listTable.Items.Add(item);
            }

            listView.Items.Clear();
            foreach (var item in viewData)
            {
                listView.Items.Add(item);
            }
        }
        #endregion

        #region 生成
        private void btnGo_Click(object sender, EventArgs e)
        {
            int a = listSelectTable.Items.Count + listSelectView.Items.Count;
            if (a == 0) return;

            List<string> names = new List<string>();
            foreach (var name in listSelectTable.Items)
            {
                names.Add(name.ToString());
            }
            foreach (var name in listSelectView.Items)
            {
                names.Add(name.ToString());
            }
            //取得所有的表、视图及字段信息
            var data = businesser.GetTableInfoForCreate(names);
            
            //开始生成
            IGenerator generator = new Generator();
            //生成根目录
            var path = Application.StartupPath + "\\CS Code\\";
            CheckDirectory(path);
            //实体类目录
            var path_model = path + "Model\\";
            CheckDirectory(path_model);
            var tt_m = Application.StartupPath + "\\Temp\\m.tt";
            //数据访问层目录
            var path_dataAccess = path + "DataAccess\\";
            CheckDirectory(path_dataAccess);
            var tt_d = Application.StartupPath + "\\Temp\\d.tt";
            //业务层目录
            var path_business = path + "Business\\";
            CheckDirectory(path_business);
            var tt_b = Application.StartupPath + "\\Temp\\b.tt";

            CreateModel(generator, tt_m, path_model, data);

            CreateDataAccess(tt_d, generator, path_dataAccess, data);

            CreateBusiness(tt_b, generator, path_business, data);

            MessageBox.Show("生成完成!" + path, "提示", MessageBoxButtons.OK);
        }

        private void CreateBusiness(string tt_b, IGenerator generator, string path_business, List<TableInfo> data)
        {
            //生成业务类
            var names_business = data.Select(p => p.Name).ToList();
            List<CreaterBusiness> data_business = new List<CreaterBusiness>();
            foreach (var item in names_business)
            {
                data_business.Add(new CreaterBusiness { TableName = item });
            }
            foreach (var item in data_business)
            {
                var tempFile = path_business + "B" + item.TableName + ".cs";
                generator.GenerateToFile(tt_b, tempFile, item);
            }
        }

        private void CreateDataAccess(string tt_d, IGenerator generator, string path_dataAccess, List<TableInfo> data)
        {
            //生成数据访问类
            var names_dataaccess = data.Select(p => p.Name).ToList();
            var file_dataaccess = path_dataAccess + "DataAccess.cs";
            var data_dataaccess = new CreaterDataAccess { TablesName = names_dataaccess };
            generator.GenerateToFile(tt_d, file_dataaccess, data_dataaccess);
        }

        private void CreateModel(IGenerator generator, string tt_m, string path_model, List<TableInfo> data)
        {
            //生成实体类
            foreach (var item in data)
            {
                var fileName = path_model + item.Name + ".cs";
                var tempData = new CreaterModel { Fields = item.Fields, TableName = item.Name };
                generator.GenerateToFile(tt_m, fileName, tempData);
            }
        }

        private void CheckDirectory(string path)
        {
            //先删除原来的
            if (Directory.Exists(path))
                Directory.Delete(path, true);

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
        }

        #endregion

        #region 表选择操作
        private void btnSelectOneTable_Click(object sender, EventArgs e)
        {
            if (listTable.SelectedItem == null) return;
            var item = listTable.SelectedItem.ToString();
            listTable.Items.RemoveAt(listTable.SelectedIndex);
            listSelectTable.Items.Add(item);
        }

        private void btnSelectAllTable_Click(object sender, EventArgs e)
        {
            List<string> data = new List<string>();
            foreach (var item in listTable.Items)
            {
                data.Add(item.ToString());
            }

            listTable.Items.Clear();

            foreach (var item in data)
            {
                listSelectTable.Items.Add(item);
            }
        }

        private void btnRemoveOneTable_Click(object sender, EventArgs e)
        {
            if (listSelectTable.SelectedItem == null) return;
            var item = listSelectTable.SelectedItem.ToString();
            listSelectTable.Items.RemoveAt(listSelectTable.SelectedIndex);
            listTable.Items.Add(item);
        }

        private void btnRemoveAllTable_Click(object sender, EventArgs e)
        {
            List<string> data = new List<string>();
            foreach (var item in listSelectTable.Items)
            {
                data.Add(item.ToString());
            }

            listSelectTable.Items.Clear();

            foreach (var item in data)
            {
                listTable.Items.Add(item);
            }
        }

        private void listTable_DoubleClick(object sender, EventArgs e)
        {
            btnSelectOneTable_Click(null, null);
        }

        private void listSelectTable_DoubleClick(object sender, EventArgs e)
        {
            btnRemoveOneTable_Click(null, null);
        }

        #endregion

        #region 视图选择操作
        private void btnSelectOneView_Click(object sender, EventArgs e)
        {
            if (listView.SelectedItem == null) return;
            var item = listView.SelectedItem.ToString();
            listView.Items.RemoveAt(listView.SelectedIndex);
            listSelectView.Items.Add(item);
        }

        private void btnSelectAllView_Click(object sender, EventArgs e)
        {
            List<string> data = new List<string>();
            foreach (var item in listView.Items)
            {
                data.Add(item.ToString());
            }

            listView.Items.Clear();

            foreach (var item in data)
            {
                listSelectView.Items.Add(item);
            }
        }

        private void btnRemoveOneView_Click(object sender, EventArgs e)
        {
            if (listSelectView.SelectedItem == null) return;
            var item = listSelectView.SelectedItem.ToString();
            listSelectView.Items.RemoveAt(listSelectView.SelectedIndex);
            listView.Items.Add(item);
        }

        private void btnRemoveAllView_Click(object sender, EventArgs e)
        {
            List<string> data = new List<string>();
            foreach (var item in listSelectView.Items)
            {
                data.Add(item.ToString());
            }

            listSelectView.Items.Clear();

            foreach (var item in data)
            {
                listView.Items.Add(item);
            }
        }
        private void listView_DoubleClick(object sender, EventArgs e)
        {
            btnSelectOneView_Click(null, null);
        }

        private void listSelectView_DoubleClick(object sender, EventArgs e)
        {
            btnRemoveOneView_Click(null, null);
        }
        #endregion


    }
}
